#! /bin/bash

# Read in the language file
source ${BBROOTDIR}/lang/${BBLANG}/burning.lang

########################## SMALL FUNCTIONS ##############################################

# A function that waits for the enter key to be pressed
wait_for_enter()
{
while true; do
        echo $bb_press_enter
        read temp
        if [[ "$temp" = "" ]]; then
                break
        else
                continue
        fi
done
}

# A function to check if overburn is enabled
check_overburn()
{
if [[ "${BBOVERBURN}" = "yes" ]]; then	# Is overburn enabled?
	BBOBURN="-overburn"		# yes it was
else
	echo $bb_no_ob			# No it wasn't
	unset BBOBURN			
fi
}

# A function to see if files in temp dir should be deleted after burning is done
check_tempdel()
{
if [[ "${BBDELTEMPBURN}" = "yes" ]]; then
	rm -rf "${BBBURNDIR}"/*
	echo "$bb_burning_tmp_1 ${BBBURNDIR} $bb_burning_tmp_1b"
	wait_for_enter
else
	echo "$bb_burning_tmp_1 ${BBBURNDIR} $bb_burning_tmp_2"
	wait_for_enter
fi
}

# A function to see if audio files should be copy protected or not
# NOTE: The files will not be copy protected, but cdrecord will write
# into TOC that files are permitted to be copied or not.
check_copy()
{
if [[ "${BBCOPY_PROTECT}" = "yes" ]]; then
	BBCOPYING="-nocopy"
else
	BBCOPYING="-copy"
fi
}

################################ AUDIO ###################################################

# a function to burn audio cds.
# it looks for wav-files in the BBBURNDIR (/tmp/burn/)..

audio_burning()
{
while read wav; do
	echo
	echo "$wav $bb_burning_audio_1"
	echo
	existing="yes"

done < <(find ${BBBURNDIR} -iname "*.[Ww][Aa][Vv]")

if [[ "$existing" != "yes" ]]; then
	echo
	echo "$bb_burning_audio_2 ${BBBURNDIR}"
else
	if [[ "${BBNORMALIZE}" = "yes" ]]; then
		cd "${BBBURNDIR}"
		normalize -m *.wav
	else
		echo $bb_burning_audio_3
	fi

	check_overburn		# Overburn enabled?
	check_copy		# Copy protected?

	if cdrecord dev=${BBCDWRITER} speed=${BBSPEED} ${BBDTAO} ${BBOPT_ONE:+"driveropts=$BBOPT_ONE"} \
	 -audio -pad -eject -v ${BBOBURN} ${BBCOPYING} ${BBBURNDIR}/*.wav; then		#Burn audio cd
		echo
		echo $bb_burning_finish_1
		check_tempdel		# See if files in temp dir should be deleted
	else
		echo $bb_burning_finish_2
		wait_for_enter
	fi
fi
}

################################# ISO #############################################################

# to burn a .iso-file in the BURNDIR.
iso_burning()
{

if [[ "$(find ${BBBURNDIR} -iname '*.[Ii][Ss][Oo]')" != "" ]]; then
	echo
	echo "ISO $bb_burning_audio_1"
	echo
	check_overburn          # Check if overburn was enabled in configure
	
	if cdrecord dev=${BBCDWRITER} speed=${BBSPEED} ${BBDTAO} ${BBOPT_ONE:+"driveropts=$BBOPT_ONE"} \
	 -eject -v ${BBOBURN} ${BBBURNDIR}/*.iso; then
		echo
		echo $bb_burning_finish_1
		check_tempdel
	else
		echo $bb_burning_finish_2
		wait_for_enter
	fi
else
	echo "$bb_burning_iso_1 ${BBBURNDIR}"
	wait_for_enter
fi
}

################################### DATA ##########################################################

# This is a function to burn all files in /tmp/burn/ as a data-CD.
# if there is an .iso-file in the BURNDIR already, it 'll execute iso_burning 
# to burn it; if there isn't one, then it'll create an iso and then execute the other function.

data_burning()
{
if [[ "$(find ${BBBURNDIR} -iname '*.[Ii][Ss][Oo]')" != "" ]]; then
	iso_burning
else
	echo; echo $bb_burning_data_1			#No ISO-file found, creating one.
	if eval "mkisofs -r -f -v -J -hide-joliet-trans-tbl -copyright \"$BBCOPYRIGHT\" \
		-A \"$BBDESCRIPTION\" -p \"$BBAUTHOR\" -publisher \"$BBPUBLISHER\" \
		-volset \"$BBNAMEOFPACKAGE\" -V \"$BBLABEL\" \
		-o ${BBBURNDIR}/BashBurn.iso ${BBBURNDIR}"; then
	iso_burning			# call function - declared above
	else
		echo $bb_burning_data_2
		echo $bb_burning_data_3
		wait_for_enter
	fi
fi
}

################################## PIPELINE ########################################################

#A function to do direct audio burning without creating wavs.
pipeline_burning()
{
#fifo counter set to 0
COUNTER=0;
FIFOLST="";
COMMANDLST="";
while read FILE; do
	#AAARG...suffixe determination is too ugly !
        if [[ "$( echo ${FILE} | sed -e 's/.*\.\(...\)$/\1/' | tr '[:lower:]' '[:upper:]'  )"x = "MP3x" ]]; then
		COUNTER=$[ ${COUNTER} + 1 ];
		FIFO="${BBFIFODIR}/FIFO$$-${COUNTER}";
		FIFOLST="${FIFOLST} ${FIFO}";
		COMMANDLST="${COMMANDLST} -audio ${FIFO}";
		mknod ${FIFO} p; #equivalent to mkfifo $FIFO
		
		#Why not to choose between L3dec and mpg123 ?
		#Oh, it's pretty simple, nobody uses the first.
		# l3dec "$FILE" "$FIFO" -sa -ign 2>/dev/null &

		exec mpg123 -qs "${FILE}" 1> "${FIFO}" &
		echo ${COUNTER-MP3} $( echo $(basename "${FILE}") | head -c35 )  flushed into ${FIFO} \(pipe\).
	elif [[ "$( echo ${FILE} | sed -e 's/.*\.\(...\)$/\1/' | tr '[:lower:]' '[:upper:]'  )"x = "WAVx" ]]; then
		COUNTER=$[ ${COUNTER} + 1 ];
		IFOF="${BBFIFODIR}/FILE$$-${COUNTER}.wav";
		FIFOLST="${FIFOLST} ${IFOF}";
		ln -s "${FILE}" ${IFOF}
		COMMANDLST="${COMMANDLST} -audio ${IFOF}";
		echo ${COUNTER-WAV} $( echo $(basename "${FILE}") | head -c35 ).
	fi
done < <(find ${BBBURNDIR} -iname \*[Mm][Pp]3 -o -iname \*]Ww][Aa][Vv])

if [[ "${COUNTER}" = "0" ]]; then
	echo "$bb_burning_fifo_1 ${BBBURNDIR}"
	wait_for_enter
fi

echo "$bb_burning_fifo_2 ${COUNTER} $bb_burning_fifo_2b"
	check_copy		#Check copy protection
	cdrecord dev=${BBCDWRITER} speed=${BBSPEED} ${BBDTAO} ${BBOPT_DRIVER:+"driver=$BBOPT_DRIVER"} \
	 fs=16m -swab -audio -pad -eject -v ${BBOBURN} ${BBCOPYING} ${COMMANDLST}
echo $bb_burning_fifo_3
	rm ${FIFOLST}
check_tempdel
}

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
case "$1" in
	"--audio")
		audio_burning
	;;
	"--data")
		data_burning
	;;
	"--iso")
		iso_burning
	;;
	"--pipeline")
		pipeline_burning
	;;
esac
